function GetTime()
	{
		return fsGetTime();
	}

function GetFrameTime()
	{
		return fsGetFrameTime();
	}

function SetupWindow(w, h, title)
	{
		return fsSetupWindow(w, h, title);
	}

function Update()
	{
		return fsUpdate();
	}

function SetupFullscreen(w, h)
	{
		return fsSetupFullscreen(w, h);
	}

function ShowMessageDialog(message, title)
	{
		return fsShowMessageDialog(message, title);
	}

function Sleep(milliseconds)
	{
		return fsSleep(milliseconds);
	}

function Shutdown()
	{
		return fsShutdown();
	}

function Print(s)
	{
		return fsPrint(s);
	}

function ReadLine()
	{
		return fsReadLine();
	}

function GetFPS()
	{
		return fsGetFPS();
	}

function MouseLock()
	{
		return fsMouseLock();
	}

function MouseDown(btn)
	{
		return fsMouseDown(btn);
	}

function JoystickButtonDown(joystick, button)
	{
		return fsJoystickButtonDown(joystick, button);
	}

function KeyDown(key)
	{
		return fsKeyDown(key);
	}

function JoystickAxisCount(joystick)
	{
		return fsJoystickAxisCount(joystick);
	}

function KeyTrigger(key)
	{
		return fsKeyTrigger(key);
	}

function JoystickButtonCount(joystick)
	{
		return fsJoystickButtonCount(joystick);
	}

function MouseY()
	{
		return fsMouseY();
	}

function MouseX()
	{
		return fsMouseX();
	}

function JoystickButtonTrigger(joystick, button)
	{
		return fsJoystickButtonTrigger(joystick, button);
	}

function MouseDeltaX()
	{
		return fsMouseDeltaX();
	}

function MouseDeltaY()
	{
		return fsMouseDeltaY();
	}

function JoystickCount()
	{
		return fsJoystickCount();
	}

function MouseTrigger(btn)
	{
		return fsMouseTrigger(btn);
	}

function JoystickAxis(joystick, axis)
	{
		return fsJoystickAxis(joystick, axis);
	}

function WriteString(filename, string)
	{
		return fsWriteString(filename, string);
	}

function ReadString(filename)
	{
		return fsReadString(filename);
	}

function ClearStencil(value)
	{
		return fsClearStencil(value);
	}

function ClearColor(r, g, b, a)
	{
		return fsClearColor(r, g, b, a);
	}

function PopFramebuffer()
	{
		return fsPopFramebuffer();
	}

function PushFramebuffer(framebuffer)
	{
		return fsPushFramebuffer(framebuffer.handle);
	}

function ClearDepth(depth)
	{
		return fsClearDepth(depth);
	}

function Clear(r, g, b, a, depth, stencil)
	{
		return fsClear(r, g, b, a, depth, stencil);
	}

